# -*- coding: utf-8 -*-
from setuphelpers import *
import requests

try:
    from setupdevhelpers import *
except:
    from bs4 import BeautifulSoup
import waptguihelper
import re
import json


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    store_url = control.sources
    if not store_url:
        store_id = waptguihelper.input_dialog(
            "Choice of app", "Enter the windows store app id (foundable in package url: https://apps.microsoft.com/store/apps)", store_url
        ).split("/")[-1]
    else:
        store_id = store_url.split("/")[-1]
    store_id = store_id.split("?")[0]
    url_ms = "https://apps.microsoft.com/store/detail/%s" % store_id
    control.sources = url_ms
    control.save_control_to_wapt()
    package_prefix = control.package.split("-")[0]
    selected = None

    # check setup.py incase the package name doesnt match the installer file
    appx_package_name = ""
    with open("setup.py", "r", encoding="utf8") as f:
        for line in f.readlines():
            if line.startswith("appx_package_name"):
                appx_package_name = line.split("=")[1].split('"')[1]
                break

    # Getting info from adguard api
    res = requests.post(
        "https://store.rg-adguard.net/api/GetFiles",
        "type=ProductId&url=%s&ring=RP&lang=fr-FR" % store_id,
        headers={"content-type": "application/x-www-form-urlencoded"},
        proxies=proxies,
    )
    all_files = []
    page = BeautifulSoup.BeautifulSoup(res.content, features="html.parser")
    for bs_search in page.find_all("a"):
        if not "BlockMap" in bs_search.text and not "eappxbundle" in bs_search.text and not "emsixbundle" in bs_search.text:
            version = bs_search.text.split("_")[1]
            bin_name = bs_search.text.replace("~", "_")
            download_url = bs_search["href"]
            pkg_splitted = re.split(r"_\d+\.", bin_name)[0]
            package_name = package_prefix + "-" + pkg_splitted.split("_")[0].replace(".", "-").lower()
            software_name = bin_name.split("_")[0].replace("-", " ").replace(".", " ")

            package_arch = get_uwp_filename_arch(bin_name, appx_package_name)
            file_dict = {
                "version": version,
                "bin_name": bin_name,
                "package_name": package_name,
                "software_name": software_name,
                "package_arch": package_arch,
                "download_url": download_url,
            }
            all_files.append(file_dict)
    if (
        "template-microsoft-store" in control.package
        or not appx_package_name
        or not len([a for a in all_files if appx_package_name in a["bin_name"]]) != 0
    ):
        if not all_files:
            error("API returned an empty list, please try again")
        selected = waptguihelper.grid_dialog(
            "Please select the proper file",
            json.dumps(all_files),
            waptguihelper.GRT_SELECTED,
            '{"columns":[{"propertyname":"version","datatype":"String","required":false,"readonly":false,"width":130},{"propertyname":"bin_name","datatype":"String","required":false,"readonly":false,"width":420},{"propertyname":"package_name","datatype":"String","required":false,"readonly":false,"width":190},{"propertyname":"software_name","datatype":"String","required":false,"readonly":false,"width":172},{"propertyname":"package_arch","datatype":"String","required":false,"readonly":false,"width":88},{"propertyname":"download_url","datatype":"String","required":false,"readonly":false,"width":1472}]}',
        )
        appx_package_name = selected[0]["bin_name"].split("_")[0]

    # Downloading files
    for dependency in all_files:
        latest_bin = dependency["bin_name"]
        if latest_bin.split(".")[-1].lower().startswith("e"):  # ignore encrypted uwp apps
            continue
        download_url = dependency["download_url"]
        if download_url.split("/")[2].endswith("microsoft.com"):
            if not isfile(latest_bin):
                print("Downloading: %s" % latest_bin)
                wget(download_url, latest_bin, proxies=proxies)
            else:
                print("Binary is present: %s" % latest_bin)
        else:
            error("ERROR: The retrieved URL will not download from microsoft's servers")

    # Import xmltodict for make_uwp_app_dict()
    if not isfile("xmltodict.py"):
        pip_download_xmltodict = "waptpython -m pip download xmltodict -d %s" % basedir
        if proxies:
            pip_download_xmltodict += " --proxy %s" % proxies["http"]
        run(pip_download_xmltodict)
        unzip(glob.glob("*.whl")[0], ".", "xmltodict.py")
        remove_file(glob.glob("*.whl")[0])
    import sys

    sys.path.append(basedir)

    # Keep app files
    uwp_app_dict = make_uwp_app_dict(appx_package_name)
    newer_version = "0"
    for uwp_app in uwp_app_dict:
        # print(uwp_app_dict[uwp_app])
        if not (
            uwp_app_dict[uwp_app]["Name"] != appx_package_name
            or uwp_app_dict[uwp_app]["TargetDeviceFamily"] == "Windows.Desktop"
            or uwp_app_dict[uwp_app]["TargetDeviceFamily"] == "Windows.Universal"
        ):
            continue
        # # Place your specific conditions for actual application here
        # if Version(uwp_app_dict[uwp_app]["Version"], 1) == Version("2019", 1):
        #     continue
        if Version(newer_version) < Version(uwp_app_dict[uwp_app]["Version"]):
            if control.max_os_version:
                if not Version(uwp_app_dict[uwp_app]["MinVersion"], 3) < Version(control.max_os_version, 3):
                    continue
            newer_uwp_app = uwp_app_dict[uwp_app]
            newer_version = newer_uwp_app["Version"]

    version = newer_uwp_app["Version"]

    # No dependencies since it will be contained
    control.depends = ""
    control.save_control_to_wapt()

    # Applying or asking control information if needed
    control.architecture = get_uwp_filename_arch(newer_uwp_app["FileName"], appx_package_name)
    control.min_os_version = ".".join(newer_uwp_app["MinVersion"].split(".")[:3])
    if not Version(newer_uwp_app["MaxVersionTested"], 3) <= Version("10.0.19041", 3):  # Ignore end-of-life Windows versions
        if (
            Version(newer_uwp_app["MinVersion"], 3) < Version("10.0.22000", 3)
            and Version(newer_uwp_app["MaxVersionTested"], 3) <= Version("10.0.22000", 3)
            and not Version(newer_uwp_app["MinVersion"]) == Version(newer_uwp_app["MaxVersionTested"])
        ):
            control.max_os_version = ".".join(newer_uwp_app["MaxVersionTested"].split(".")[:3])
    else:
        control.max_os_version = ""
    ask_control_categories()
    ask_control_package(
        selected[0]["package_name"] if selected is not None else control.package,
        "template-microsoft-store",
        remove_base_files=True,
    )
    ask_control_name(selected[0]["software_name"] if selected is not None else control.name, "Template Microsoft Store")
    ask_control_description("update_package")

    # Changing setup.py appx_package_name variable
    new_lines = []
    with open("setup.py", "r", encoding="utf8") as f:
        for line in f.readlines():
            if line.startswith("appx_package_name"):
                line = 'appx_package_name = "%s"\n' % appx_package_name
            new_lines.append(line)
    with open("setup.py", "w", encoding="utf8") as f:
        f.writelines(new_lines)

    # Get icon.png
    if "b4:Dependencies" in newer_uwp_app:
        unzip(newer_uwp_app["FileName"], ".", newer_uwp_app["b4:Dependencies"][0]["FileName"], False)
        icon_png = unzip(newer_uwp_app["b4:Dependencies"][0]["FileName"], ".", "*.targetsize-48.png", False)
        shutil.move(icon_png[0], "WAPT\\icon.png")
    else:
        icon_png = unzip(newer_uwp_app["FileName"], ".", "*.targetsize-48.png", False)
        shutil.move(icon_png[0], "WAPT\\icon.png")
    [remove_file(a) for a in glob.glob("*.png")]

    # Placing binaries in a dir ["appxbundle", "msixbundle", "appx", "msix]
    bins_dir = control.package.split("-", 1)[1]
    if isdir(bins_dir):
        remove_tree(bins_dir)
    mkdirs(bins_dir)
    if not isfile(makepath(bins_dir, newer_uwp_app["FileName"])):
        shutil.move(newer_uwp_app["FileName"], bins_dir)
    for l in [glob.glob(a["Name"] + "_*") for a in uwp_app_dict[newer_uwp_app["FileName"]]["Dependencies"]]:
        for f in l:
            if not isfile(makepath(bins_dir, f)):
                shutil.move(f, bins_dir)

    # Removing remaining files ["appxbundle", "msixbundle", "appx", "msix]
    for f in glob.glob("*.appxbundle") + glob.glob("*.msixbundle") + glob.glob("*.appx") + glob.glob("*.msix"):
        remove_file(f)

    # Changing version of the package
    if Version(version) > Version(control.get_software_version()):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Validating or not update-package-sources
    return package_updated


def make_uwp_app_dict(appx_package_name):
    import xmltodict

    ms_app_db = {}
    # for ms_app_file in (
    #     glob.glob("*.appxbundle")
    #     + glob.glob("*.msixbundle")
    #     + glob.glob("*.appx")
    #     + glob.glob("*.msix")
    # ):
    for ms_app_file in (
        glob.glob(f"{appx_package_name}*.appxbundle")
        + glob.glob(f"{appx_package_name}*.msixbundle")
        + glob.glob(f"{appx_package_name}*.appx")
        + glob.glob(f"{appx_package_name}*.msix")
    ):
        manifest = unzip(ms_app_file, ".", "AppxManifest.xml", False)
        bundle_manifest = unzip(ms_app_file, ".", "AppxMetadata\AppxBundleManifest.xml", False)
        if not manifest:
            before_dependencies = []
            if bundle_manifest:
                with open(bundle_manifest[0], encoding="utf8") as xml_file:
                    data_dict = xmltodict.parse(xml_file.read(), attr_prefix="")
                ms_app_info = {
                    "FileName": ms_app_file,
                    "Name": data_dict["Bundle"]["Identity"]["Name"],
                    "Version": data_dict["Bundle"]["Identity"]["Version"],
                }

                for app_pkg in data_dict["Bundle"]["Packages"]["Package"]:
                    if app_pkg["Type"] == "application":
                        if not "b4:Dependencies" in app_pkg:
                            ms_app_info.update({"TargetDeviceFamily": None})
                            before_dependency_info = {
                                "FileName": app_pkg["FileName"],
                                "Version": app_pkg["Version"],
                                "Architecture": app_pkg["Architecture"],
                                # "MinVersion": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["MinVersion"],
                                # "MaxVersionTested": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["MaxVersionTested"],
                            }
                            before_dependencies.append(before_dependency_info)
                            # ms_app_info.update(
                            #     {"MinVersion": before_dependency_info["MinVersion"], "MaxVersionTested": before_dependency_info["MaxVersionTested"]}
                            # )

                            continue
                        else:
                            if not "b4:TargetDeviceFamily" in app_pkg["b4:Dependencies"]:
                                ms_app_info.update({"TargetDeviceFamily": None})
                                continue
                            else:
                                if type(app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]) == list:
                                    if (
                                        sum(
                                            1
                                            for t in app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]
                                            if t["Name"] in ["Windows.Desktop", "Windows.Universal"]
                                        )
                                        == 0
                                    ):
                                        ms_app_info.update(
                                            {
                                                "TargetDeviceFamily": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"][0]["Name"],
                                                "MinVersion": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"][0]["MinVersion"],
                                                "MaxVersionTested": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"][0]["MaxVersionTested"],
                                            }
                                        )
                                        continue
                                    else:
                                        app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"] = [
                                            a
                                            for a in app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]
                                            if a["Name"] == "Windows.Desktop" or a["Name"] == "Windows.Universal"
                                        ][0]
                                        ms_app_info.update(
                                            {
                                                "TargetDeviceFamily": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["Name"],
                                                "MinVersion": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["MinVersion"],
                                                "MaxVersionTested": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["MaxVersionTested"],
                                            }
                                        )

                            before_dependency_info = {
                                "FileName": app_pkg["FileName"],
                                "Version": app_pkg["Version"],
                                "Architecture": app_pkg["Architecture"],
                                "TargetDeviceFamily": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["Name"],
                                "MinVersion": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["MinVersion"],
                                "MaxVersionTested": app_pkg["b4:Dependencies"]["b4:TargetDeviceFamily"]["MaxVersionTested"],
                            }
                            before_dependencies.append(before_dependency_info)
                            ms_app_info.update(
                                {
                                    "TargetDeviceFamily": before_dependency_info["TargetDeviceFamily"],
                                    "MinVersion": before_dependency_info["MinVersion"],
                                    "MaxVersionTested": before_dependency_info["MaxVersionTested"],
                                }
                            )

                if before_dependencies:
                    for dependency in before_dependencies:
                        unzip(
                            ms_app_file,
                            ".",
                            makepath(
                                dependency["FileName"],
                            ),
                            False,
                        )
                        manifest = unzip(dependency["FileName"], ".", makepath("AppxManifest.xml"), False)
                        if isfile(dependency["FileName"]):
                            remove_file(dependency["FileName"])
                        with open(manifest[0], encoding="utf8") as xml_file:
                            dependency_data_dict = xmltodict.parse(xml_file.read(), attr_prefix="")

                    sub_dependencies = []
                    if type(dependency_data_dict["Package"]["Dependencies"]["PackageDependency"]) == list:
                        for sub_app_pkg in dependency_data_dict["Package"]["Dependencies"]["PackageDependency"]:
                            sub_dependency_info = {
                                "Name": sub_app_pkg["Name"],
                                "MinVersion": sub_app_pkg["MinVersion"],
                            }
                            sub_dependencies.append(sub_dependency_info)
                    else:
                        sub_dependency_info = {
                            "Name": dependency_data_dict["Package"]["Dependencies"]["PackageDependency"]["Name"],
                            "MinVersion": dependency_data_dict["Package"]["Dependencies"]["PackageDependency"]["MinVersion"],
                        }
                        sub_dependencies.append(sub_dependency_info)
                    ms_app_info["b4:Dependencies"] = before_dependencies
                    ms_app_info["Dependencies"] = sub_dependencies
                    if ms_app_info["TargetDeviceFamily"] is not None and not ms_app_info["TargetDeviceFamily"]:
                        ms_app_info.update(
                            {
                                "TargetDeviceFamily": dependency_data_dict["Package"]["Dependencies"]["TargetDeviceFamily"]["Name"],
                                "MinVersion": dependency_data_dict["Package"]["Dependencies"]["TargetDeviceFamily"]["MinVersion"],
                                "MaxVersionTested": dependency_data_dict["Package"]["Dependencies"]["TargetDeviceFamily"]["MaxVersionTested"],
                            }
                        )
                ms_app_db[ms_app_file] = ms_app_info
            else:
                error("nothing to parse")
        else:
            manifest = unzip(ms_app_file, ".", makepath("AppxManifest.xml"), False)
            with open(manifest[0], encoding="utf8") as xml_file:
                simple_data_dict = xmltodict.parse(xml_file.read(), attr_prefix="")

            simple_dependencies = []

            for key, value in simple_data_dict["Package"]["Dependencies"].items():
                if key == "PackageDependency":
                    for simple_dependency in value:
                        simple_dependency_info = {
                            "Name": simple_dependency["Name"],
                            "MinVersion": simple_dependency["MinVersion"],
                        }
                        simple_dependencies.append(simple_dependency_info)
                if key == "TargetDeviceFamily":
                    simple_ms_app_info = {
                        "FileName": ms_app_file,
                        "Name": simple_data_dict["Package"]["Identity"]["Name"],
                        "Version": simple_data_dict["Package"]["Identity"]["Version"],
                        "Architecture": simple_data_dict["Package"]["Identity"]["ProcessorArchitecture"],
                        "TargetDeviceFamily": value["Name"],
                        "MinVersion": value["MinVersion"],
                        "MaxVersionTested": value["MaxVersionTested"],
                    }

            simple_ms_app_info["Dependencies"] = simple_dependencies
            ms_app_db[ms_app_file] = simple_ms_app_info

        for xml_file in glob.glob("AppxManifest.xml") + glob.glob("AppxBundleManifest.xml"):
            remove_file(xml_file)

    return ms_app_db


def get_uwp_filename_arch(appx_filename, appx_package_name=None):
    if not appx_package_name:
        appx_package_name = None
    if "arm64" in appx_filename:
        return "arm64"
    elif "arm" in appx_filename:
        return "arm"
    elif "x64" in appx_filename:
        return "x64"
    if appx_filename is not None and "x86" in appx_filename and (appx_package_name is None or not appx_package_name in appx_filename):
        return "x86"
    else:
        return "all"


def ask_control_description(blank_str=None):
    """Requesting that the user supply package description for the control.description field

    Args:
        blank_str   (str): The description will be cleared if it includes the specified string to avoid using the template description (default: do not clear description)
    """
    if not control.description:
        control.description = waptguihelper.input_dialog("Description", "Please fill the description", control.description)
        control.save_control_to_wapt()
    if blank_str is not None and blank_str in control.description:
        control.description = ""
        control.save_control_to_wapt()
    return control.description


def ask_control_categories():
    """Requesting that the user supply package categories for the control.categories field if empty or Template is selected"""

    if control.categories == "" or control.categories == "Template":
        categories = waptguihelper.grid_dialog(
            "Select package categories",
            [
                "Internet",
                "Utilities",
                "Messaging",
                "Security",
                "System and network",
                "Media",
                "Development",
                "Office",
                "Drivers",
                "Education",
                "Configuration",
                "CAD",
                "Template",
                "Dependency",
                "Extension",
            ],
            waptguihelper.GRT_SELECTED,
        )
        if categories:
            control.categories = ",".join([a["unknown"] for a in categories])
        else:
            control.categories = ""
        control.save_control_to_wapt()
    return control.categories


def ask_control_package(control_package=control.package, conditionnal_package_name=None, remove_base_files=False):
    """Requesting that the user provide a package name to be entered into the control.package field, and offering the possibility of removing the base files (icon.png and changelog.txt) for template package usage

    Args:
        control_package             (str)   : prefilled control_package (default: actual control_package)
        conditionnal_package_name   (str)   : only ask when the control.package contains conditionnal_package_name (default: always ask for control_package)
        remove_base_files           (bool)  : removes base files if parameter is True and conditionnal_package_name is provided (default: False)
    """
    if conditionnal_package_name is None or conditionnal_package_name in control.package:
        control.package = waptguihelper.input_dialog(
            control.package,
            "You can redefine the package name",
            control_package.replace(conditionnal_package_name, "") if conditionnal_package_name is not None else control_package,
        )
        control.save_control_to_wapt()

    # Removing template files
    if conditionnal_package_name in control.package and remove_base_files:
        if isfile("WAPT\\changelog.txt"):
            remove_file("WAPT\\changelog.txt")
        if isfile("WAPT\\icon.png"):
            remove_file("WAPT\\icon.png")
    return control.package


def ask_control_name(control_name=control.name, conditionnal_package_name=None):
    """Requesting that the user provide a package name to be entered into control.name field

    Args:
        control_name                (str)   : prefilled control_name (default: control.name)
        conditionnal_package_name   (str)   : only ask when the control.name contains conditionnal_package_name (default: always ask for control_name)
    """
    if conditionnal_package_name is None or conditionnal_package_name in control.name:
        control.name = waptguihelper.input_dialog(control.name, "You can redefine the name for the self-service", control_name)
        control.save_control_to_wapt()
    return control.name
